/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2017  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef GRIDWIDGETHEADERFILE
#define GRIDWIDGETHEADERFILE

#include <QGraphicsView>
#include <QObject>
#include <QSize>
#include <QPoint>
#include <vector>
#include <QPixmap>

class TGridWidgetScene;
class QGraphicsProxyWidget;
class QTimeLine;
class QWheelEvent;
class QMouseEvent;

class TGridWidget : public QGraphicsView
{
Q_OBJECT

public:
// CREATORS
	TGridWidget(const QWidget *Parent=0);
	virtual ~TGridWidget(void);

// ACCESSORS
	int ActiveWidgetIndex(void) const {return d_CurrentActiveWidgetIndex;}
	QWidget* ActiveWidget(void) const;	// returns null if no widget is active


// MANIPULATORS
	void AddWidget(QWidget *Widget,Qt::WindowFlags Flags = 0);
	void RemoveWidget(const QWidget *Widget);

	void Resize_Widgets(void);				// WARNING.  Only call this following resize_event of parent
	void ActivateWidget(const int Index);
	void ActivateWidget(const QWidget *Widget);

signals:
	void Grid_Item_Selected(const QWidget*);	// only sent if multiple views displayed
	void Animation_Complete(void);

private slots:
	void AnimationValueChanged(qreal);
	void AnimationFinished(void);

protected:
	virtual void wheelEvent(QWheelEvent *Event);
	virtual void mousePressEvent(QMouseEvent *event);

private:

	TGridWidgetScene					*d_GraphicsScene;
	QTimeLine							*d_AnimationTimeLine;
	bool								d_AnimationRunning;
	std::vector<QGraphicsProxyWidget*>	d_Widgets;

	QSize								d_WidgetSize;
	int									d_MaxColumns;

	QRectF								d_current_viewport;
	QRectF								d_target_viewport;

// ActivateWidget can only be done after initial resizing.  Call is therefore queued
	bool								d_InitialResizingDone;
	bool								d_ActivateWidgetQueued;
	int									d_InitialActivateWidget;
	int									d_CurrentActiveWidgetIndex;

	bool								d_delayed_remove_widget_active;
	std::vector<const QWidget*>			d_delayed_remove_widgets;

	void Delayed_RemoveWidget(void);
	void Create_Background_Image(void);


// NOT IMPLEMENTED
	TGridWidget& operator=(const TGridWidget&);

};


class TGridWidgetScene : public QGraphicsScene
{
Q_OBJECT
public:
// CREATORS
	TGridWidgetScene(const QObject *parent=0);
	virtual ~TGridWidgetScene(void);

// ACCESSORS

// MANIPULATORS
	void Set_Animation_Mode(const bool state) {d_animation_mode = state;}
	void Set_Background_Image(const QPixmap &image) {d_Scene_Image = image;}

protected:
	virtual void drawForeground (QPainter * painter,const QRectF & rect);
	virtual void drawItems(QPainter *painter, int numItems, QGraphicsItem *items[], const QStyleOptionGraphicsItem options[], QWidget *widget);

private:
	bool								d_animation_mode;
	QPixmap								d_Scene_Image;

// NOT IMPLEMENTED
	TGridWidgetScene(const TGridWidgetScene&);
	TGridWidgetScene& operator=(const TGridWidgetScene&);

};

#endif
