/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2017  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QWidget>
#include <QString>
#include <QFocusEvent>
#include <QKeyEvent>

#include "valueedit.h"

TDoubleEdit::TDoubleEdit(
	const QWidget						*parent)
:QLineEdit(const_cast<QWidget*>(parent))
{
	d_display_precision = 1;
	d_value_changed = false;
	d_value = 0.0;
	
	connect(this,SIGNAL(textEdited(const QString&)),this,SLOT(Text_Edited(const QString&)));
}

TDoubleEdit::TDoubleEdit(
	const double						&value,
	const QWidget						*parent)
:QLineEdit(const_cast<QWidget*>(parent))
{
	d_display_precision = 1;
	d_value = value;
	
	this->Update_Display();
	d_value_changed = false;
	
	connect(this,SIGNAL(textEdited(const QString&)),this,SLOT(Text_Edited(const QString&)));
}

TDoubleEdit::~TDoubleEdit(void)
{
	
	disconnect(this,SIGNAL(textEdited(const QString&)),0,0);

}

void TDoubleEdit::Set_Display_Precision(
	const int							precision)
{
	if(precision < 0)
	{
		d_display_precision = 0;
	}
	else if(precision > 15)
	{
		d_display_precision = 15;
	}
	else
	{
		d_display_precision = precision;
	}
	
	this->Update_Display();
}

void TDoubleEdit::Set_Value(
	const double						&value)
{
	d_value_changed = false;
	d_value = value;
	
	this->Update_Display();
}

void TDoubleEdit::Text_Edited(
	const QString						&text)
{
	double								value;
	
	value = text.toDouble();
	
	if(value != d_value)
	{
		d_value = value;
		d_value_changed = true;
	}
}

void TDoubleEdit::focusOutEvent(
	QFocusEvent							*event)
{
	if(d_value_changed)
	{
		emit Edit_Finished();
	}
	
	d_value_changed = false;
	
	QLineEdit::focusOutEvent(event);
}

void TDoubleEdit::keyPressEvent(
	QKeyEvent							*event)
{
	int									key(event->key());
	
	if(key == Qt::Key_Return || key == Qt::Key_Enter)
	{		
		if(d_value_changed)
		{
			emit Edit_Finished();
		}
		
		d_value_changed = false;
	}
	
	QLineEdit::keyPressEvent(event);
}

void TDoubleEdit::Update_Display(void)
{
	this->setText(QString("%1").arg(d_value,0,'f',d_display_precision));
}


TIntEdit::TIntEdit(
	const QWidget						*parent)
:QLineEdit(const_cast<QWidget*>(parent))
{
	d_value_changed = false;
	d_value = 0;

	connect(this,SIGNAL(textEdited(const QString&)),this,SLOT(Text_Edited(const QString&)));
}

TIntEdit::TIntEdit(
	const int							value,
	const QWidget						*parent)
:QLineEdit(const_cast<QWidget*>(parent))
{
	d_value = value;
	
	this->Update_Display();
	d_value_changed = false;
	
	connect(this,SIGNAL(textEdited(const QString&)),this,SLOT(Text_Edited(const QString&)));
}

TIntEdit::~TIntEdit(void)
{
	
	disconnect(this,SIGNAL(textEdited(const QString&)),0,0);
	
}

void TIntEdit::Set_Value(
	const int							value)
{
	d_value_changed = false;
	d_value = value;
	
	this->Update_Display();
}

void TIntEdit::Text_Edited(
	const QString						&text)
{
	double								value;
	
	value = text.toDouble();
	
	if(value != d_value)
	{
		if(value < 0)
		{
			d_value = static_cast<int>(value - 0.5);
		}
		else
		{
			d_value = static_cast<int>(value + 0.5);
		}
		
		d_value_changed = true;
	}
}

void TIntEdit::focusOutEvent(
	QFocusEvent							*event)
{
	if(d_value_changed)
	{
		emit Edit_Finished();
	}
	
	d_value_changed = false;
	
	QLineEdit::focusOutEvent(event);
}

void TIntEdit::keyPressEvent(
	QKeyEvent							*event)
{
	int									key(event->key());
	
	if(key == Qt::Key_Return || key == Qt::Key_Enter)
	{
		if(d_value_changed)
		{
			emit Edit_Finished();
		}
		
		d_value_changed = false;
	}
	
	QLineEdit::keyPressEvent(event);
}

void TIntEdit::Update_Display(void)
{
	this->setText(QString("%1").arg(d_value));
}

