/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2017  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef VALUEEDITHEADERFILE
#define VALUEEDITHEADERFILE

#include <QLineEdit>

class QFocusEvent;
class QKeyEvent;

class TDoubleEdit : public QLineEdit
{
Q_OBJECT
public:
	
// CREATORS
	TDoubleEdit(const QWidget *parent=0);
	TDoubleEdit(const double& value, const QWidget *parent=0);
	virtual ~TDoubleEdit(void);
	
// ACCESSORS
	int Display_Precision(void) const {return d_display_precision;}
	double Value(void) const {return d_value;}
	
// MANIPULATORS
	void Set_Display_Precision(const int precision);
	void Set_Value(const double &value);
	
signals:
	void Edit_Finished(void);
	
private slots:
	void Text_Edited(const QString&);
	
protected:
	virtual void focusOutEvent(QFocusEvent *event);
	virtual void keyPressEvent(QKeyEvent *event);
	
private:
	double								d_value;
	int									d_display_precision;
	bool								d_value_changed;
	
	void Update_Display(void);
	
// NOT IMPLEMENTED
	TDoubleEdit(const TDoubleEdit&);
	TDoubleEdit& operator=(const TDoubleEdit&);
};

class TIntEdit : public QLineEdit
{
	Q_OBJECT
public:
	
	// CREATORS
	TIntEdit(const QWidget *parent=0);
	TIntEdit(const int value, const QWidget *parent=0);
	virtual ~TIntEdit(void);
	
	// ACCESSORS
	int Value(void) const {return d_value;}
	
	// MANIPULATORS
	void Set_Value(const int value);
	
signals:
	void Edit_Finished(void);
	
private slots:
	void Text_Edited(const QString&);
	
protected:
	virtual void focusOutEvent(QFocusEvent *event);
	virtual void keyPressEvent(QKeyEvent *event);
	
private:
	int									d_value;
	bool								d_value_changed;
	
	void Update_Display(void);
	
	// NOT IMPLEMENTED
	TIntEdit(const TIntEdit&);
	TIntEdit& operator=(const TIntEdit&);
};


#endif
