/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2017  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef INTERPOLATIONEDITORHEADERFILE
#define INTERPOLATIONEDITORHEADERFILE

#include <QMainWindow>
#include <vector>

#include "../../core/vector2.h"

class QGroupBox;
class QToolButton;
class QSpinBox;
class QComboBox;
class QSettings;
class QCloseEvent;

class TIntEdit;
class TDoubleEdit;
class TEditTable;
class TInterpolationGraphDialog;

class TInterpolationEditor : public QMainWindow
{
Q_OBJECT

public:

// CREATORS
	TInterpolationEditor(const QWidget *parent=0, Qt::WindowFlags flags=0);
	virtual ~TInterpolationEditor(void);

// ACCESSORS

// MANIPULATORS

signals:

private slots:
	void Display_Precision_Changed(int);
	void Copy_Output(void);
	void Show_Hide_Graph(bool);
	void Graph_Closed(void);
	void Zero_Offset_Toggled(bool);
	
	void Input_Selection_Mode_Changed(void);
	void Input_Range_Changed(void);
	void Input_Increment_Count_Changed(void);
	void Input_Changed(const int,const int,const double&);

	void Edit_Selection_Mode_Changed(void);
	void Edit_Range_Changed(void);
	void Edit_Increment_Count_Changed(void);
	void Edit_Changed(const int,const int,const double&);
	
	void Start_Multiple_Cell_Change(void);
	void End_Multiple_Cell_Change(void);
	
protected:
	virtual void closeEvent(QCloseEvent *event);

private:
	
enum TCopyOption
{
	COPY_DATA_ONLY=0,
	COPY_POSITION_AND_DATA
};
	
	QComboBox							*d_input_selection_combo;
	QComboBox							*d_edit_selection_combo;
	QComboBox							*d_output_copy_option_combo;
	QGroupBox							*d_edit_group;
	QGroupBox							*d_input_group;
	QGroupBox							*d_output_group;
	TIntEdit							*d_input_count_edit;
	TIntEdit							*d_edit_count_edit;
	TIntEdit							*d_output_count_edit;
	TDoubleEdit							*d_edit_end_edit;
	TDoubleEdit							*d_edit_increment_edit;
	TDoubleEdit							*d_edit_start_edit;
	TDoubleEdit							*d_input_end_edit;
	TDoubleEdit							*d_input_increment_edit;
	TDoubleEdit							*d_input_start_edit;
	TDoubleEdit							*d_output_end_edit;
	TDoubleEdit							*d_output_increment_edit;
	TDoubleEdit							*d_output_start_edit;
	TEditTable							*d_edit_table;
	TEditTable							*d_input_table;
	TEditTable							*d_output_table;
	QSpinBox							*d_precision_spin;
	QToolButton							*d_graph_toggle_button;
	QToolButton							*d_output_copy_button;
	QToolButton							*d_zero_offset_check;
	
	TInterpolationGraphDialog			*d_interpolation_graph_dialog;
	
	QSettings							*d_settings;
	
	std::vector<TVector2>				d_input_data;
	std::vector<TVector2>				d_edit_data;
	std::vector<TVector2>				d_output_data;
	
	bool								d_multicell_active;
	
	
	void Initialize_Input_Output_Editor(void);

	void Initialize_Edit_Editor(void);
	void Update_Edit_Editor(void);
	
	void Update_Output_Editor(void);
	void Update_Graph(void);

// NOT IMPLEMENTED
	TInterpolationEditor(const TInterpolationEditor&);
	TInterpolationEditor& operator=(const TInterpolationEditor&);
};

#endif
