/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2017  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#include <QApplication>
#include <QGridLayout>
#include <QWidget>
#include <QCloseEvent>
#include <QColor>

#include "../../core/graphwidget.h"

#include "interpolationgraphdialog.h"

TInterpolationGraphDialog::TInterpolationGraphDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QGridLayout							*grid_layout;


	this->resize(500,400);
	
	this->setMinimumWidth(400);
	this->setMinimumHeight(300);


	grid_layout = new QGridLayout(this);

	d_graph_widget = new TGraphWidget(this);
	grid_layout->addWidget(d_graph_widget,0,0,1,1);

	this->setWindowTitle(QStringLiteral("Graph Data"));
	
	d_graph_widget->Title(QStringLiteral("Input Data: Blue  Output Data:Red"));
	d_graph_widget->VerticalLegend(QStringLiteral("Value"));
	d_graph_widget->HorizontalLegend(QStringLiteral("Position"));
}

TInterpolationGraphDialog::~TInterpolationGraphDialog(void)
{
}


void TInterpolationGraphDialog::closeEvent(
	QCloseEvent							*event)
{
	emit Dialog_Close();
	event->accept();
}

void TInterpolationGraphDialog::Update(
	const std::vector<TVector2>			&input,
	const std::vector<TVector2>			&output)
{
	int									set;
	std::vector<TVector2>::const_iterator iter;
	
	d_graph_widget->DeleteDataSets();
	
	set = d_graph_widget->CreateDataSet(QColor(0,0,164));
	
	for(iter = input.begin();iter != input.end();++iter)
	{
		d_graph_widget->AddPoint(set,(*iter));
	}
	
	set = d_graph_widget->CreateDataSet(QColor(164,0,0));
	
	for(iter = output.begin();iter != output.end();++iter)
	{
		d_graph_widget->AddPoint(set,(*iter));
	}
	
	d_graph_widget->UpdateGraph();

}

